import pytest

from aiodeu.cetl import get_field, get_conditional_data, explode_list


TEST_DATA = {
    "header": {
        "1": "One",
        "2": [
            {
                "f": "Three",
                "a": "Five"
            },
            {
                "f": "Four",
                "a": "Six"
            }
        ]
    }
}


@pytest.mark.parametrize("test_input,expected", [
    ("header.1", "One"),
    ("header.1.4", "One"),
    ("header.4", "")
])
def test_get_field(test_input, expected):
    assert get_field(test_input, TEST_DATA) == expected


def test_get_conditional_data():
    items = get_field("header.2", TEST_DATA)
    assert get_conditional_data(items, "f", "Four") == {'a': 'Six', 'f': 'Four'}


def test_explode_list():
    assert explode_list(TEST_DATA["header"], "2") == [
        {'1': 'One', '2.f': 'Three', '2.a': 'Five'},
        {'1': 'One', '2.f': 'Four', '2.a': 'Six'}
    ]



import logging
import os
import ssl
from typing import Type

import faust
from faust.serializers import codecs

from aiodeu.codecs import AvroJsonCodec
from aiodeu.config import Config
from aiodeu.utils import write_to_file

logger = logging.getLogger(__name__)


def create_app(config: Type[Config], faust_app_kwargs: dict = {}) -> faust.App:
    app_kwargs = {
        "broker": config.BROKER_LIST,
        "value_serializer": "AvroJsonCodec",
        "store": "memory://",
        "topic_replication_factor": 3,
        "topic_partitions": 12,
        "topic_allow_declare": False,
        "topic_disable_leader": True,
        "consumer_auto_offset_reset": "earliest",
        # "stream_wait_empty": False
    }
    cert_verify = faust_app_kwargs.pop("cert_verify", True)
    app_kwargs.update(faust_app_kwargs)
    codecs.register('AvroJsonCodec', AvroJsonCodec(config=config))

    if config.BROKER_CERT:
        cert = write_to_file(os.path.join(config.BASE_DIR, "client.cert"), config.BROKER_CERT)
        key = write_to_file(os.path.join(config.BASE_DIR, "client.key"), config.BROKER_KEY)
        if cert_verify:
            ssl_context = ssl.create_default_context(purpose=ssl.Purpose.SERVER_AUTH)
        else:
            ssl_context = ssl._create_unverified_context(purpose=ssl.Purpose.SERVER_AUTH)
        ssl_context.load_cert_chain(cert, keyfile=key)
        ssl_context.load_cert_chain(cert, keyfile=key)
        app_kwargs.update({
            "broker_credentials": ssl_context
        })

    new_app = faust.App(
        config.APP_NAME,
        **app_kwargs
    )
    return new_app



from typing import Any, Type

from faust.serializers import codecs
from schema_registry.client import SchemaRegistryClient
from schema_registry.serializers import AvroMessageSerializer

from aiodeu.config import Config


class AvroJsonCodec(codecs.Codec):
    def __init__(self, *args, config: Type[Config] = Config, **kwargs: Any) -> None:
        self.config = config
        self.client = SchemaRegistryClient(self.config.AVRO_SCHEMA_REGISTRY)
        self.message_serializer = AvroMessageSerializer(self.client)
        super().__init__(*args, **kwargs)

    """Schema registry /schemas/ids/1263"""
    def _dumps(self, obj: dict, schema_id: int = 1) -> bytes:
        """Not used in application but for testing sending avro binary to kafka"""
        avro_schema = self.client.get_by_id(schema_id)
        return self.message_serializer.encode_record_with_schema(
            subject=self.config.TOPIC_NAME,
            schema=avro_schema,
            record=obj
        )

    def _loads(self, s: bytes) -> Any:
        """Load avro binary and return reader iterable"""
        return self.message_serializer.decode_message(s)



import io
import logging
import tempfile
from typing import Union

import boto3
from botocore.exceptions import ClientError

from aiodeu.config import Config
from aiodeu.utils import async_wrap

logger = logging.getLogger(__name__)


class StorageException(Exception):
    """Exception raised when file read errors"""
    pass


class S3Transport:
    def __init__(self, aws_access_key_id, aws_secret_access_key, bucket_name, region=Config.AWS_S3_REGION, sse="aws:kms"):
        self.resource = boto3.resource(
            "s3",
            region,
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key
        )
        self.bucket = self.resource.Bucket(bucket_name)
        self.sse = sse

    async def write(self, file_contents: Union[str, io.BytesIO], key: str) -> None:
        logger.info(f"Sending file to s3: {key}")
        if isinstance(file_contents, str):
            bytes_data = io.BytesIO()
            bytes_data.write(file_contents.encode("utf-8"))
        elif isinstance(file_contents, io.BytesIO):
            bytes_data = file_contents
        else:
            raise Exception("file_contents must be str or io.BytesIO")
        bytes_data.seek(0)
        upload_fileobj = async_wrap(self.bucket.upload_fileobj)
        await upload_fileobj(
            bytes_data, key,
            ExtraArgs={"ServerSideEncryption": self.sse}
        )

    async def read(self, file_path: str) -> Union[str, bytes]:
        """Returns file contents"""
        with tempfile.NamedTemporaryFile("w+b") as file_obj:
            try:
                download_file = async_wrap(self.bucket.download_file)
                await download_file(
                    file_path,
                    file_obj.name
                )
            except ClientError:
                raise StorageException(f"Can't download file: {file_path}")

            try:
                with open(file_obj.name, "r") as contents:
                    return contents.read()
            except UnicodeDecodeError:
                with open(file_obj.name, "rb") as contents:
                    return contents.read()



import json

import pytest

from aiodeu.codecs import AvroJsonCodec
from aiodeu.utils import BytesJsonEncoder


@pytest.mark.asyncio()
async def test_loads(registry_server, test_message_serializer, test_avro_data, test_client):
    serializer = AvroJsonCodec()
    serializer.message_serializer = test_message_serializer
    serializer.client = test_client
    schema_id = registry_server.schema_id
    for avro_data in test_avro_data:
        in_data = json.loads(json.dumps(avro_data, cls=BytesJsonEncoder))
        serialised = serializer._dumps(obj=in_data, schema_id=schema_id)
        out_data = serializer._loads(serialised)
        assert out_data == in_data



#!/usr/bin/env python
import argparse
import asyncio
import json
import os

try:
    from avro.datafile import DataFileReader
    from avro.io import DatumReader
except ImportError:
    print("You need to install the dev requirements")

from aiodeu.utils import BytesJsonEncoder


def read_avro(file_path):
    return DataFileReader(open(file_path, "rb"), DatumReader())


def get_path(path: str) -> str:
    return os.path.abspath(os.path.join(os.getcwd(), path))


async def avro_to_json(in_path: str, out_path: str = "") -> None:
    reader = read_avro(get_path(in_path))
    rows = [record for record in reader]
    print(json.dumps(rows, cls=BytesJsonEncoder))
    if out_path:
        with open(get_path(out_path), "+w") as f:
            f.write(json.dumps(rows, cls=BytesJsonEncoder))


async def print_avro_schema(in_path: str, out_path: str = "") -> None:
    reader = read_avro(get_path(in_path))
    schema = reader.meta
    print(json.dumps(schema))
    if out_path:
        with open(get_path(out_path), "+w") as f:
            f.write(json.dumps(schema))


async def run(args):
    if args.func == "avro_to_json":
        await avro_to_json(args.in_path, args.out_path)
    if args.func == "print_avro_schema":
        await print_avro_schema(args.in_path, args.out_path)
    else:
        print("Must specify one of 'avro_to_json' or 'print_avro_schema'")


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("func", metavar="Function", type=str, nargs=1, help="Function to run",
                        choices=("avro_to_json", "print_avro_schema"))
    parser.add_argument("-i", "--in", dest="in_path", type=str, default="")
    parser.add_argument("-o", "--out", dest="out_path", type=str, default="")
    args = parser.parse_args()

    loop = asyncio.get_event_loop()
    loop.run_until_complete(run(args))



import os


REG_ENDPOINT = os.environ.get("AVRO_SCHEMA_REGISTRY", "http://127.0.0.1:8080").rstrip()


class Config:
    DEBUG: bool = False
    APP_NAME: str = os.environ.get("APP_NAME", "app_consumer").rstrip()
    BASE_DIR: str = os.getcwd()
    TOPIC_NAME: str = os.environ.get("TOPIC_NAME", "test").rstrip()
    BROKER: str = os.environ.get("BROKER", "localhost:9092").rstrip()
    BROKER_LIST: list = [f"kafka://{b}" for b in os.environ.get("BROKER", "localhost:9092").split(",")]
    BROKER_HOSTS: list = os.environ.get("BROKER", "localhost:9092").split(",")
    ZOOKEEPER: list = os.environ.get("ZOOKEEPER", "localhost").split(",")
    BROKER_KEY: str = os.environ.get("BROKER_KEY", "")
    BROKER_CERT: str = os.environ.get("BROKER_CERT", "")
    BROKER_GROUP_ID: str = os.environ.get("BROKER_GROUP_ID", "").rstrip()
    AVRO_SCHEMA_REGISTRY: str = f"http://{REG_ENDPOINT}" if not REG_ENDPOINT.startswith("http") else REG_ENDPOINT
    AWS_ACCESS_KEY_ID: str = os.environ.get("AWS_ACCESS_KEY_ID", "").rstrip()
    AWS_SECRET_ACCESS_KEY: str = os.environ.get("AWS_SECRET_ACCESS_KEY", "").rstrip()
    AWS_S3_BUCKET_NAME: str = os.environ.get("AWS_S3_BUCKET_NAME", "test").rstrip()
    KEY_PREFIX: str = os.environ.get("KEY_PREXIX", "test/").rstrip()
    AWS_S3_REGION: str = os.environ.get("AWS_S3_REGION", "eu-west-2").rstrip()



import json

from fastavro import reader
import pytest


@pytest.mark.asyncio()
async def test_app(test_agent, test_avro_data, test_avro_bytes, s3_transport):
    async with test_agent.test_context() as agent:
        await agent.put(reader(test_avro_bytes))
        key = "test.json"
        s3_bucket = s3_transport.bucket
        assert [s.key for s in s3_bucket.objects.all()] == [key]
        assert json.loads(await s3_transport.read(key)) == test_avro_data



# -*- coding: utf-8 -*-
from Cython.Build import cythonize
from setuptools import setup
from setuptools import Extension
from setuptools.dist import Distribution
from distutils.command.build_ext import build_ext
import os

packages = ["aiodeu"]

package_data = {"": ["*"]}

extras_require = {
    "aws": ["boto3>=1.26.20,<2.0.0"],
    "faust": [
        "faust-streaming[cython,fast]>=0.10.4,<0.11.0",
        "python-schema-registry-client>=2.4.1,<3.0.0",
    ],
}

entry_points = {"console_scripts": ["aiodeu = aiodeu.console:main"]}

setup_kwargs = {
    "name": "aiodeu",
    "version": "0.1.30",
    "description": "aio data engineering utils",
    "long_description": "None",
    "author": "Josh Rowe",
    "author_email": "s-block@users.noreply.github.com",
    "maintainer": "Josh Rowe",
    "maintainer_email": "s-block@users.noreply.github.com",
    "url": "https://github.com/s-block/aiodeu",
    "packages": packages,
    "package_data": package_data,
    "extras_require": extras_require,
    "entry_points": entry_points,
    "python_requires": ">=3.8.1,<4.0",
}


try:
    from Cython.Build import cythonize
    extensions = ["aiodeu/cetl.pyx"]
    # gcc arguments hack: enable optimizations
    os.environ["CFLAGS"] = "-O3"
    # Build
    setup_kwargs.update(
        {
            "ext_modules": cythonize(
                extensions,
                language_level=3,
                compiler_directives={"linetrace": True},
            ),
            "cmdclass": {"build_ext": build_ext},
        }
    )
except ImportError:
    pass

setup(**setup_kwargs)
